r"""
Provides frozendict, a simple immutable dictionary.
"""

try:   # pragma: no cover
    from ._frozendict import *
    c_ext = True
    # noinspection PyUnresolvedReferences
    del _frozendict
except ImportError:
    from ._frozendict_py import *
    c_ext = False

from .version import version as __version__
from . import monkeypatch
from .cool import *
from . import cool


def _getFrozendictJsonEncoder(BaseJsonEncoder = None):
    if BaseJsonEncoder is None:  # pragma: no cover
        from json.encoder import JSONEncoder
        
        BaseJsonEncoder = JSONEncoder
    
    class FrozendictJsonEncoderInternal(BaseJsonEncoder):
        def default(self, obj):
            if isinstance(obj, frozendict):  # pragma: no cover
                # TODO create a C serializer
                return dict(obj)
            
            return BaseJsonEncoder.default(
                self,
                obj
            )  # pragma: no cover
    
    return FrozendictJsonEncoderInternal


FrozendictJsonEncoder = _getFrozendictJsonEncoder()
monkeypatch.patchOrUnpatchAll(patch = True, warn = False)


from collections.abc import Mapping

# noinspection PyUnresolvedReferences
Mapping.register(frozendict)
del Mapping


if c_ext:  # pragma: no cover
    __all__ = (frozendict.__name__, )
else:
    __all__ = _frozendict_py.__all__
    del _frozendict_py

# TODO deprecated, to remove in future versions
FrozenOrderedDict = frozendict

__all__ += cool.__all__
__all__ += (FrozendictJsonEncoder.__name__, "FrozenOrderedDict")



import json

import frozendict as cool
import pytest
from frozendict import frozendict
from frozendict.monkeypatch import MonkeypatchWarning


class A:
    pass


@pytest.fixture
def object_to_serialize():
    return frozendict()


@pytest.fixture
def object_to_serialize_2():
    return A()


@pytest.fixture
def serialized_object():
    return "{}"


def test_get_json_encoder(
    object_to_serialize,
    object_to_serialize_2,
    serialized_object,
):
    if cool.c_ext:
        cool.monkeypatch.patchOrUnpatchJson(patch=True)
    else:
        with pytest.warns(MonkeypatchWarning):
            cool.monkeypatch.patchOrUnpatchJson(patch=True, warn=True)
    
    assert json.dumps(object_to_serialize) == serialized_object
    
    with pytest.raises(TypeError):
        json.dumps(object_to_serialize_2)
    
    cool.monkeypatch.patchOrUnpatchJson(patch=False, warn=False)
    
    if cool.c_ext:
        with pytest.raises(TypeError):
            json.dumps(object_to_serialize)
